//Server_BuildZones

//Solved problems
//---------------
//==> Bricks can be placed into the air
//==> Non-admins w/ trust can destroy existing zones
//==> Non-admins can place their own zones under certain conditions
//==> Bricks can be placed partly in zones
//==> Trust checking prevents people with no trust from building in someone else's zone
//==> Float error plays to everyone in the server
//==> Bricks in multiple zones can't be planted

//Major problems
//--------------
//==> Wanding leaves floating bricks
//==> Hammering leaves floating bricks

//Minor problems
//--------------
//==> Can hold down plant in air, then jump on result
//==> AlwaysShowWireframe sometimes shows wireframe on other bricks
//==> Placing ghost, then removing zone still allows for planting that single brick
//==> Bricks created by other (server side) scripts don't work
//==> Can't place zones over non-trusted bricks
//==> Can't undo after trying to plant floating bricks

if(forceRequiredAddOn("Brick_Large_Cubes") == $Error::AddOn_NotFound)
	return;
	
if(isFile("Add-Ons/System_ReturnToBlockland/server.cs"))
{
	if(!$BuildZones::PrefsSet)
	{
		if(!$RTB::RTBR_ServerControl_Hook)
			exec("Add-Ons/System_ReturnToBlockland/RTBR_ServerControl_Hook.cs");
		RTB_registerPref("No building outside of zones","Build Zones","BuildZone::Mode","bool","Server_BuildZones",1,0,0);
		$BuildZones::PrefsSet = true;
	}
}
else
{
	// If you do not have RTB you can change the preferences here
	$BuildZone::Mode = 1; //Do not allow players to build outside of zones (1 - True, 0 - False)
}

if(isPackage(BuildZone))
	deactivatePackage(BuildZone);
package BuildZone
{
	function ServerCmdPlantBrick(%client)
	{
		if(%client.isAdmin)
			return Parent::ServerCmdPlantBrick(%client);
		%brick = %client.player.tempBrick;
		%data = %brick.getDataBlock();
		if(%data.isBuildZone || %data.isAntiBuildZone)
		{
			messageClient(%client,'',"\c6You are not allowed to build this brick");
			return;
		}
		initContainerBoxSearch(%brick.getPosition(),vectorSub(getBrickDim(%brick),"-0.001 -0.001 -0.001"),$TypeMasks::FxBrickAlwaysObjectType);
		while(%n = ContainerSearchNext())
		{
			if(!isObject(%n))
				continue;
			%z = %n.getDataBlock();
			if(%z.isAntiBuildZone)
			{
				messageClient(%client,'',"\c6You are not allowed to build in this zone");
				return;
			}
			else if(%z.isBuildZone && isInCompoundBuildZone(%brick.getWorldBox()))
				%bordersBZ = true;
		}
		if(!%bordersBZ && $BuildZone::Mode)
		{
			messageClient(%client,'',"\c6You are not allowed to build outside of a build zone");
			return;
		}
		else
			return Parent::ServerCmdPlantBrick(%client);
	}
	
	function fxDTSBrick::onPlant(%brick)
	{
		if(!needsTrustNoZone(%brick))
		{
			initContainerBoxSearch(%brick.getPosition(),vectorSub(getBrickDim(%brick),"-0.001 -0.001 -0.001"),$TypeMasks::FxBrickAlwaysObjectType);
			while(%n = ContainerSearchNext())
			{
				if(%n.getDataBlock().isBuildZone)
				{
					%trustLevel = getTrustLevel(%brick,%n);
					if(%trustLevel >= 1)
						continue;
					%zoneBL_ID = %n.client.bl_id;
					%brickBL_ID = %brick.client.bl_id;
					setMutualBrickGroupTrust(%brickBL_ID,%zoneBL_ID,1);
					schedule(0,0,"setMutualBrickGroupTrust",%brickBL_ID,%zoneBL_ID,%trustLevel);
				}
			}
		}
		if(isZoneFloating(%brick))
		{
			%brick.deadSoon = true;
			%brick.schedule(0,delete);
			messageClient(%brick.client,'MsgPlantError_Float');
		}
		return Parent::onPlant(%brick);
	}
	
	function hammerImage::OnHitObject(%a,%player,%b,%col,%c,%d)
	{
		%client = %player.client;
		if(%col.getType() & $Typemasks::FxBrickAlwaysObjectType)
		{
			%data = %col.getDataBlock();
			if(!%client.isAdmin && (%data.isBuildZone || %data.isAntiBuildZone))
			{
				messageClient(%client,'',"\c6You are not allowed to destroy this");
				return;
			}
		}
		return parent::OnHitObject(%a,%player,%b,%col,%c,%d);
	}
	
	function FxDTSBrick::OnRemove(%brick)
	{
		return parent::onRemove(%brick);
		for(%i=0;%i<%brick.getNumUpBricks();%i++)
		{
			%b = %brick.getUpBrick(%i);
			if(isFloatingNoZone(%brick))
				%b.kill();
		}
		for(%i=0;%i<%brick.getNumDownBricks();%i++)
		{
			%b = %brick.getDownBrick(%i);
			if(isFloatingNoZone(%brick))
				%b.kill();
		}
	}
};
activatePackage(BuildZone);

function isInCompoundBuildZone(%box1)
{
	initContainerBoxSearch(getBoxCenter(%box1),getBoxDim(%box1,-0.001),$TypeMasks::FxBrickAlwaysObjectType);
	while(%n = ContainerSearchNext())
	{
		if(!isObject(%n) || !%n.getDataBlock().isBuildZone)
			continue;
		%box2 = %n.getWorldBox();
		%b1Ymax = getWord(%box1,4);
		%b1Zmin = getWord(%box1,2);
		%b1Zmax = getWord(%box1,5);
		%b2Xmin = getWord(%box2,0);
		%b2Xmax = getWord(%box2,3);
		%b2Ymin = getWord(%box2,1);
		%b2Zmin = getWord(%box2,2);
		%b2Zmax = getWord(%box2,5);
		if(
		(getWord(%box1,0) < %b2Xmin && !isInCompoundBuildZone(getWords(%box1,0,2) SPC %b2Xmin SPC getWords(%box1,4,5))) ||
		(%b2Xmax < getWord(%box1,3) && !isInCompoundBuildZone(%b2Xmax SPC getWords(%box2,1,2) SPC getWords(%box1,3,5))) ||
		(getWord(%box1,1) < %b2Ymin && !isInCompoundBuildZone(%b2Xmin SPC getWords(%box1,1,2) SPC %b2Xmax SPC %b2Ymin SPC %b1Zmax)) ||
		(getWord(%box2,4) < %b1Ymax && !isInCompoundBuildZone(%b2Xmin SPC %b1Ymax SPC %b1Zmin SPC %b2Xmax SPC getWords(%box1,4,5))) ||
		(%b1Zmin < %b2Zmin && !isInCompoundBuildZone(getWords(%box2,0,1) SPC %b1Zmin SPC getWords(%box2,3,4) SPC %b2Zmin)) ||
		(%b2Zmax < %b1Zmax && !isInCompoundBuildZone(getWords(%box2,0,1) SPC %b2Zmax SPC getWords(%box2,3,4) SPC %b1Zmax)))
			continue;
		%var = true;
		break;
	}
	if(%var)
		return true;
}

function getBoxDim(%b,%offset)
{
    %x = getWord(%b,3) - getWord(%b,0) + %offset;
    %y = getWord(%b,4) - getWord(%b,1) + %offset;
    %z = getWord(%b,5) - getWord(%b,2) + %offset;
    return %x SPC %y SPC %z;
}

function getBoxInBoxFromBox(%box1,%box2)
{
	for(%i=0;%i<6;%i++)
	{
		if(%i<3)
			%nBox = %nBox SPC mClampMax(getWord(%box1,%i),getWord(%box2,%i));
		else
			%nBox = %nBox SPC mClampMin(getWord(%box1,%i),getWord(%box2,%i));
	}
	return trim(%nBox);
}

function mClampMin(%n1,%n2)
{
	if(%n1 <= %n2)
		return %n1;
	return %n2;
}

function mClampMax(%n1,%n2)
{
	if(%n1 >= %n2)
		return %n1;
	return %n2;
}

function isFloatingNoZoneSetUp(%root,%brick)
{
	if(%brick.isBaseplate || %brick.isFloating)
	{
		%root.isZoneFloating = true;
		for(%i=0;%i<getWordCount($possiFloat[%root]);%i++)
			getWord($possiFloat[%root],%i).isZoneFloating = true;
		return;
	}
	for(%i=0;%i<%brick.getNumUpBricks();%i++)
	{
		%b = %brick.getUpBrick(%i);
		if(!%b.getDataBlock().isWaterBrick && !%b.deadSoon)
			$possiFloat[%root].add();
	}
	for(%i=0;%i<%brick.getNumDownBricks();%i++)
	{
		%b = %brick.getDownBrick(%i);
		if(!%b.getDataBlock().isWaterBrick && !%b.deadSoon)
			return false;
	}
	return true;
}

function getAllConnectingNoZone(%brick)
{
    
}

function needsTrustNoZone(%brick)
{
    for(%i=0;%i<%brick.getNumUpBricks();%i++)
	{
		%b = %brick.getUpBrick(%i);
		if(%b.getDataBlock().isWaterBrick)
			continue;
		else if(getTrustLevel(%b,%brick) < 1)
			return true;
	}
	for(%i=0;%i<%brick.getNumDownBricks();%i++)
	{
		%b = %brick.getDownBrick(%i);
		if(%b.getDataBlock().isWaterBrick)
			continue;
		else if(getTrustLevel(%b,%brick) < 1)
			return true;
	}
	return false;
}

function switchBoxFormat(%var1,%var2) //If only %var1 exists, the box is in (Min SPC Max) form, but if %var2 exists the box is in (Center,Dimensions) form
{
	if(%var2 !$= "")
	{
		for(%i=0;%i<3;%i++)
		{
			%c = %c SPC (getWord(%var2,%i) / 2);
			echo(%c);
		}
		%c = trim(%c);
		echo(%c);
		return vectorSub(%var1,%c) SPC vectorAdd(%var1,%c);
	}
	else
	{
		for(%i=0;%i<3;%i++)
			%c = %c SPC getWord(%var1,%i + 3) - getWord(%var1,%i);
		return vectorAdd(vectorScale(trim(%c),0.5),getWords(%var1,0,2)) TAB trim(%c);
	}
}

function isBoxInsideBox(%box1,%box2) //Uses (Min SPC Max) form
{
	for(%i=0;%i<6;%i++)
	{
		%b = getWord(%box1,%i);
		%j = %i;
		if(%i >= 3)
			%j = %i - 3;
		if(mClampF(%b,getWord(%box2,%j),getWord(%box2,%j + 3)) != %b)
			return false;
	}
	return true;
}

function isBoxInfringeBox(%box1,%box2)
{
	for(%i=0;%i<3;%i++)
	{
		%j = %i + 3;
		if(getWord(%box1,%i) >= getWord(%box2,%j) || getWord(%box2,%i) >= getWord(%box1,%j))
			return false;
	}
	return true;
}

function isZoneFloating(%brick)
{
    if(%brick.getDataBlock().isWaterBrick || %brick.isBaseplate)
        return false;
    for(%i=0;%i<%brick.getNumUpBricks();%i++)
    {
		%b = %brick.getUpBrick(%i);
        if(!%b.getDataBlock().isWaterBrick && !%b.deadSoon)
            return false;
    }
    for(%i=0;%i<%brick.getNumDownBricks();%i++)
    {
		%b = %brick.getDownBrick(%i);
        if(!%b.getDataBlock().isWaterBrick && !%b.deadSoon)
            return false;
    }
    return true;
}

function getBrickDim(%brick,%offset)
{
    %b = %brick.getWorldBox();
    %x = getWord(%b,3) - getWord(%b,0) + %offset;
    %y = getWord(%b,4) - getWord(%b,1) + %offset;
    %z = getWord(%b,5) - getWord(%b,2) + %offset;
    return %x SPC %y SPC %z;
}

function setZoneProperties(%brick)
{
	%brick.setColliding(0);
	%brick.setRendering(0);
	%brick.setRayCasting(0);
}

//Build Zone Bricks
if(!isObject(brickBuildZone64xData))
{
	datablock fxDTSBrickData(brickBuildZone64xData : brick64xCubeData)
	{
		category = "Special";
		subCategory = "Zones";
		uiName = "Build Zone 64x";
		isBuildZone = true;
		isWaterBrick = true;
		alwaysShowWireFrame = false;
	};
}

function brickBuildZone64xData::OnAdd(%data,%brick)
{
	Parent::OnAdd(%data,%brick);
	setZoneProperties(%brick);
}

if(!isObject(brickBuildZone32xData))
{
	datablock fxDTSBrickData(brickBuildZone32xData : brick32xCubeData)
	{
		category = "Special";
		subCategory = "Zones";
		uiName = "Build Zone 32x";
		isBuildZone = true;
		isWaterBrick = true;
		alwaysShowWireFrame = false;
	};
}

function brickBuildZone32xData::OnAdd(%data,%brick)
{
	Parent::OnAdd(%data,%brick);
	setZoneProperties(%brick);
}

if(!isObject(brickBuildZone16xData))
{
	datablock fxDTSBrickData(brickBuildZone16xData : brick16xCubeData)
	{
		category = "Special";
		subCategory = "Zones";
		uiName = "Build Zone 16x";
		isBuildZone = true;
		isWaterBrick = true;
		alwaysShowWireFrame = false;
	};
}

function brickBuildZone16xData::OnAdd(%data,%brick)
{
	Parent::OnAdd(%data,%brick);
	setZoneProperties(%brick);
}

//Anti-Build Zone Bricks
if(!isObject(brickAntiBuildZone64xData))
{
	datablock fxDTSBrickData(brickAntiBuildZone64xData : brick64xCubeData)
	{
		category = "Special";
		subCategory = "Zones";
		uiName = "Anti-Build Zone 64x";
		isAntiBuildZone = true;
		isWaterBrick = true;
		alwaysShowWireFrame = false;
	};
}

function brickAntiBuildZone64xData::OnAdd(%data,%brick)
{
	Parent::OnAdd(%data,%brick);
	setZoneProperties(%brick);
}

if(!isObject(brickAntiBuildZone32xData))
{
	datablock fxDTSBrickData(brickAntiBuildZone32xData : brick32xCubeData)
	{
		category = "Special";
		subCategory = "Zones";
		uiName = "Anti-Build Zone 32x";
		isAntiBuildZone = true;
		isWaterBrick = true;
		alwaysShowWireFrame = false;
	};
}

function brickAntiBuildZone32xData::OnAdd(%data,%brick)
{
	Parent::OnAdd(%data,%brick);
	setZoneProperties(%brick);
}

if(!isObject(brickAntiBuildZone16xData))
{
	datablock fxDTSBrickData(brickAntiBuildZone16xData : brick16xCubeData)
	{
		category = "Special";
		subCategory = "Zones";
		uiName = "Anti-Build Zone 16x";
		isAntiBuildZone = true;
		isWaterBrick = true;
		alwaysShowWireFrame = false;
	};
}

function brickAntiBuildZone16xData::OnAdd(%data,%brick)
{
	Parent::OnAdd(%data,%brick);
	setZoneProperties(%brick);
}

